@@ -163,9 +163,12 @@ static void msg_read_thread(){
         // make sure that the message recieved is compatible
         // and that we're not trying to use variables that don't exist (yet)
         size_t msg_size = msgrcv(msgid, (void *) raw_msg, sizeof(raw_msg), 1, 0);
-        if (msg_size != -1)
+        if (msg_size != size_t(-1))
         {
             if (hdr->version == 1){
+                if (msg_size > offsetof(struct mangoapp_msg_v1, pid))
+                    HUDElements.g_gamescopePid = mangoapp_v1->pid;
+
                 if (msg_size > offsetof(struct mangoapp_msg_v1, visible_frametime_ns)){
                     bool should_new_frame = false;
                     if (mangoapp_v1->visible_frametime_ns != ~(0lu) && (!params.no_display || logger->is_active())) {
@@ -285,6 +288,8 @@ static bool render(GLFWwindow* window) {
 
 int main(int, char**)
 {
+    XInitThreads();
+
     // Setup window
     glfwSetErrorCallback(glfw_error_callback);
     if (!glfwInit())
@@ -324,11 +329,13 @@ int main(int, char**)
     window_size = ImVec2(params.width, params.height);
     deviceName = (char*)glGetString(GL_RENDERER);
     sw_stats.deviceName = deviceName;
-
+    SPDLOG_DEBUG("mangoapp deviceName: {}", deviceName);
     #define GLX_RENDERER_VENDOR_ID_MESA 0x8183
     auto pfn_glXQueryCurrentRendererIntegerMESA = (Bool (*)(int, unsigned int*)) (glfwGetProcAddress("glXQueryCurrentRendererIntegerMESA"));
-    if (pfn_glXQueryCurrentRendererIntegerMESA) {
+    // This will return 0x0 vendorID on NVIDIA so just go to else
+    if (pfn_glXQueryCurrentRendererIntegerMESA && vendorID != 0x0) {
         pfn_glXQueryCurrentRendererIntegerMESA(GLX_RENDERER_VENDOR_ID_MESA, &vendorID);
+        SPDLOG_DEBUG("mangoapp vendorID: {:#x}", vendorID);
     } else {
         if (deviceName.find("Radeon") != std::string::npos
         || deviceName.find("AMD") != std::string::npos){
@@ -341,7 +348,6 @@ int main(int, char**)
     }
 
     HUDElements.vendorID = vendorID;
-    init_gpu_stats(vendorID, 0, params);
     init_system_info();
     sw_stats.engine = EngineTypes::GAMESCOPE;
     std::thread(msg_read_thread).detach();
@@ -353,25 +359,25 @@ int main(int, char**)
                     PropModeReplace, (unsigned char *)&value, 1);
     // Main loop
     while (!glfwWindowShouldClose(window)){
+        check_keybinds(params);
+
         if (!params.no_display){
             if (mangoapp_paused){
-                glfwRestoreWindow(window);
+                glfwShowWindow(window);
                 uint32_t value = 1;
                 XChangeProperty(x11_display, x11_window, overlay_atom, XA_CARDINAL, 32, PropModeReplace, (unsigned char *)&value, 1);
                 XSync(x11_display, 0);
                 mangoapp_paused = false;
-                {
-                    amdgpu_run_thread = true;
-                    amdgpu_c.notify_one();
-                }
+                // resume all GPU threads
+                if (gpus)
+                    for (auto gpu : gpus->available_gpus)
+                        gpu->resume();
             }
             {
                 std::unique_lock<std::mutex> lk(mangoapp_m);
                 mangoapp_cv.wait(lk, []{return new_frame || params.no_display;});
                 new_frame = false;
             }
-
-            check_keybinds(params, vendorID);
             // Start the Dear ImGui frame
             {
                 if (render(window)) {
@@ -398,17 +404,27 @@ int main(int, char**)
 
             glfwSwapBuffers(window);
         } else if (!mangoapp_paused) {
-            glfwIconifyWindow(window);
+            glfwHideWindow(window);
             uint32_t value = 0;
             XChangeProperty(x11_display, x11_window, overlay_atom, XA_CARDINAL, 32, PropModeReplace, (unsigned char *)&value, 1);
             XSync(x11_display, 0);
             mangoapp_paused = true;
-            {
-                amdgpu_run_thread = false;
-                amdgpu_c.notify_one();
-            }
-            std::unique_lock<std::mutex> lk(mangoapp_m);
-            mangoapp_cv.wait(lk, []{return !params.no_display;});
+            // pause all GPUs threads
+            if (gpus)
+                for (auto gpu : gpus->available_gpus)
+                    gpu->pause();
+
+            // If mangoapp is hidden, using mangoapp_cv.wait() causes a hang.
+            // Because of this hang, we can't detect if the user presses R_SHIFT + F12,
+            // which prevents mangoapp from being unhidden.
+            // To address this, replace mangoapp_cv.wait() with sleep().
+            //
+            // If severe power usage issues arise, find an alternative solution.
+
+            // std::unique_lock<std::mutex> lk(mangoapp_m);
+            // mangoapp_cv.wait(lk, []{return !params.no_display;});
+        } else {
+            usleep(100000);
         }
     }
 
