@@ -23,6 +23,7 @@ static  std::vector<std::string> blacklist {
     "Battle.net.exe",
     "BethesdaNetLauncher.exe",
     "EpicGamesLauncher.exe",
+    "halloy",
     "IGOProxy.exe",
     "IGOProxy64.exe",
     "monado-service",
@@ -56,7 +57,10 @@ static  std::vector<std::string> blacklist {
     "rundll32.exe",
     "Launcher", //Paradox Interactive Launcher
     "steamwebhelper.exe",
-    "EpicWebHelper.exe"
+    "EpicWebHelper.exe",
+    "UplayWebCore.exe",
+    "plutonium.exe",
+    "plutonium-launcher-win32.exe"
 };
 
 
