@@ -24,6 +24,9 @@ int glXQueryDrawable(void *dpy, void* glxdraw, int attr, unsigned int * value);
 int64_t glXSwapBuffersMscOML(void* dpy, void* drawable, int64_t target_msc, int64_t divisor, int64_t remainder);
 
 unsigned int eglSwapBuffers( void*, void* );
+void* eglGetPlatformDisplay( unsigned int, void*, const intptr_t* );
+void* eglGetDisplay( void* );
+void* eglGetProcAddress( const char* );
 
 #ifdef __cplusplus
 }
