@@ -25,6 +25,7 @@
 #endif
 #include "amdgpu.h"
 #include "fps_metrics.h"
+#include "fex.h"
 
 #define CHAR_CELSIUS    "\xe2\x84\x83"
 #define CHAR_FAHRENHEIT "\xe2\x84\x89"
@@ -184,76 +185,77 @@ void HudElements::version(){
 }
 
 void HudElements::gpu_stats(){
+    if (!gpus)
+        gpus = std::make_unique<GPUS>(HUDElements.params);
+
+    size_t i = 0;
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_stats]){
-        ImguiNextColumnFirstItem();
-        const char* gpu_text;
-        if (HUDElements.params->gpu_text.empty())
-            gpu_text = "GPU";
-        else
-            gpu_text = HUDElements.params->gpu_text.c_str();
-        HUDElements.TextColored(HUDElements.colors.gpu, "%s", gpu_text);        ImguiNextColumnOrNewRow();
-        auto text_color = HUDElements.colors.text;
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_load_change]){
-            struct LOAD_DATA gpu_data = {
-                HUDElements.colors.gpu_load_low,
-                HUDElements.colors.gpu_load_med,
-                HUDElements.colors.gpu_load_high,
-                HUDElements.params->gpu_load_value[0],
-                HUDElements.params->gpu_load_value[1]
-            };
+        for (auto& gpu : gpus->selected_gpus()) {
+            ImguiNextColumnFirstItem();
+            HUDElements.TextColored(HUDElements.colors.gpu, "%s", gpu->gpu_text().c_str());
 
-            auto load_color = change_on_load_temp(gpu_data, gpu_info.load);
-            right_aligned_text(load_color, HUDElements.ralign_width, "%i", gpu_info.load);
-            ImGui::SameLine(0, 1.0f);
-            HUDElements.TextColored(load_color,"%%");
-        }
-        else {
-            right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.load);
-            ImGui::SameLine(0, 1.0f);
-            HUDElements.TextColored(text_color,"%%");
-            // ImGui::SameLine(150);
-            // ImGui::Text("%s", "%");
-        }
-
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_temp]){
             ImguiNextColumnOrNewRow();
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
-                right_aligned_text(text_color, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu_info.temp));
-            else
-                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.temp);
-            ImGui::SameLine(0, 1.0f);
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact])
-                HUDElements.TextColored(HUDElements.colors.text, "°");
-            else
+            auto text_color = HUDElements.colors.text;
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_load_change]){
+                struct LOAD_DATA gpu_data = {
+                    HUDElements.colors.gpu_load_low,
+                    HUDElements.colors.gpu_load_med,
+                    HUDElements.colors.gpu_load_high,
+                    HUDElements.params->gpu_load_value[0],
+                    HUDElements.params->gpu_load_value[1]
+                };
+
+                auto load_color = change_on_load_temp(gpu_data, gpu->metrics.load);
+                right_aligned_text(load_color, HUDElements.ralign_width, "%i", gpu->metrics.load);
+                ImGui::SameLine(0, 1.0f);
+                HUDElements.TextColored(load_color,"%%");
+            }
+            else {
+                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.load);
+                ImGui::SameLine(0, 1.0f);
+                HUDElements.TextColored(text_color,"%%");
+                // ImGui::SameLine(150);
+                // ImGui::Text("%s", "%");
+            }
+
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_temp]){
+                ImguiNextColumnOrNewRow();
+                if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu->metrics.temp));
+                else
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.temp);
+                ImGui::SameLine(0, 1.0f);
+                if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact])
+                    HUDElements.TextColored(HUDElements.colors.text, "°");
+                else
+                    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
+                        HUDElements.TextColored(HUDElements.colors.text, "°F");
+                    else
+                        HUDElements.TextColored(HUDElements.colors.text, "°C");
+            }
+
+            if (gpu->metrics.junction_temp > -1 && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_junction_temp]) {
+                ImguiNextColumnOrNewRow();
+                if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu->metrics.junction_temp));
+                else
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.junction_temp);
+                ImGui::SameLine(0, 1.0f);
                 if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
                     HUDElements.TextColored(HUDElements.colors.text, "°F");
                 else
                     HUDElements.TextColored(HUDElements.colors.text, "°C");
-        }
-
-        if (gpu_info.junction_temp > -1 && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_junction_temp]) {
-            ImguiNextColumnOrNewRow();
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
-                right_aligned_text(text_color, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu_info.junction_temp));
-            else
-                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.junction_temp);
-            ImGui::SameLine(0, 1.0f);
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
-                HUDElements.TextColored(HUDElements.colors.text, "°F");
-            else
-                HUDElements.TextColored(HUDElements.colors.text, "°C");
-            ImGui::SameLine(0, 1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "Jnc");
-            ImGui::PopFont();
-        }
-
-        if (HUDElements.vendorID == 0x1002 || HUDElements.vendorID == 0x10de){
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_fan] && cpuStats.cpu_type != "APU"){
-                ImguiNextColumnOrNewRow();
-                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.fan_speed);
                 ImGui::SameLine(0, 1.0f);
-                if (gpu_info.fan_rpm) {
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                HUDElements.TextColored(HUDElements.colors.text, "Jnc");
+                ImGui::PopFont();
+            }
+
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_fan] && !gpu->is_apu()){
+                ImguiNextColumnOrNewRow();
+                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.fan_speed);
+                ImGui::SameLine(0, 1.0f);
+                if (gpu->metrics.fan_rpm) {
                     ImGui::PushFont(HUDElements.sw_stats->font1);
                     HUDElements.TextColored(HUDElements.colors.text, "RPM");
                 } else {
@@ -264,38 +266,41 @@ void HudElements::gpu_stats(){
                 }
                 ImGui::PopFont();
             }
-        }
 
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_core_clock]){
-            ImguiNextColumnOrNewRow();
-            right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.CoreClock);
-            ImGui::SameLine(0, 1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "MHz");
-            ImGui::PopFont();
-        }
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_core_clock]){
+                ImguiNextColumnOrNewRow();
+                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.CoreClock);
+                ImGui::SameLine(0, 1.0f);
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                HUDElements.TextColored(HUDElements.colors.text, "MHz");
+                ImGui::PopFont();
+            }
 
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_power]) {
-            ImguiNextColumnOrNewRow();
-            char str[16];
-            snprintf(str, sizeof(str), "%.1f", gpu_info.powerUsage);
-            if (strlen(str) > 4)
-                right_aligned_text(text_color, HUDElements.ralign_width, "%.0f", gpu_info.powerUsage);
-            else
-                right_aligned_text(text_color, HUDElements.ralign_width, "%.1f", gpu_info.powerUsage);
-            ImGui::SameLine(0, 1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "W");
-            ImGui::PopFont();
-        }
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_power]) {
+                ImguiNextColumnOrNewRow();
+                char str[16];
+                snprintf(str, sizeof(str), "%.1f", gpu->metrics.powerUsage);
+                if (strlen(str) > 4)
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%.0f", gpu->metrics.powerUsage);
+                else
+                    right_aligned_text(text_color, HUDElements.ralign_width, "%.1f", gpu->metrics.powerUsage);
+                ImGui::SameLine(0, 1.0f);
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                HUDElements.TextColored(HUDElements.colors.text, "W");
+                ImGui::PopFont();
+            }
 
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_voltage]) {
-            ImguiNextColumnOrNewRow();
-            right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.voltage);
-            ImGui::SameLine(0, 1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "mV");
-            ImGui::PopFont();
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_voltage]) {
+                ImguiNextColumnOrNewRow();
+                right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu->metrics.voltage);
+                ImGui::SameLine(0, 1.0f);
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                HUDElements.TextColored(HUDElements.colors.text, "mV");
+                ImGui::PopFont();
+            }
+            if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_horizontal])
+                ImGui::TableNextRow();
+            i++;
         }
     }
 }
@@ -490,42 +495,62 @@ void HudElements::io_stats(){
 }
 
 void HudElements::vram(){
+    if (!gpus)
+        gpus = std::make_unique<GPUS>(HUDElements.params);
+
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_vram]){
-        ImguiNextColumnFirstItem();
-        HUDElements.TextColored(HUDElements.colors.vram, "VRAM");
-        ImguiNextColumnOrNewRow();
-        // Add gtt_used to vram usage for APUs
-        if (cpuStats.cpu_type == "APU")
-            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu_info.memoryUsed + gpu_info.gtt_used);
-        else
-            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu_info.memoryUsed);
-        if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact]){
-            ImGui::SameLine(0,1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "GiB");
-            ImGui::PopFont();
-        }
+        size_t i = 0;
+        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_stats]){
+            for (auto& gpu : gpus->selected_gpus()) {
+                ImguiNextColumnFirstItem();
+                // Just iterate through the user selected GPUs
+                if (!HUDElements.params->gpu_list.empty())
+                    for (auto& gpu_index : HUDElements.params->gpu_list)
+                        if (gpu_index < gpus->available_gpus.size())
+                            if (i != gpu_index)
+                                continue;
 
-        if (gpu_info.memory_temp > -1 && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_temp]) {
-            ImguiNextColumnOrNewRow();
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
-                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu_info.memory_temp));
-            else
-                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu_info.memory_temp);
-            ImGui::SameLine(0, 1.0f);
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
-                HUDElements.TextColored(HUDElements.colors.text, "°F");
-            else
-                HUDElements.TextColored(HUDElements.colors.text, "°C");
-        }
 
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_clock]){
-            ImguiNextColumnOrNewRow();
-            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu_info.MemClock);
-            ImGui::SameLine(0, 1.0f);
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            HUDElements.TextColored(HUDElements.colors.text, "MHz");
-            ImGui::PopFont();
+                HUDElements.TextColored(HUDElements.colors.vram, gpu->vram_text().c_str());
+
+                ImguiNextColumnOrNewRow();
+                // Add gtt_used to vram usage for APUs
+                if (gpu->is_apu())
+                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu->metrics.memoryUsed + gpu->metrics.gtt_used);
+                else
+                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu->metrics.memoryUsed);
+                if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact]){
+                    ImGui::SameLine(0,1.0f);
+                    ImGui::PushFont(HUDElements.sw_stats->font1);
+                    HUDElements.TextColored(HUDElements.colors.text, "GiB");
+                    ImGui::PopFont();
+                }
+
+                if (gpu->metrics.memory_temp > -1 && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_temp]) {
+                    ImguiNextColumnOrNewRow();
+                    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", HUDElements.convert_to_fahrenheit(gpu->metrics.memory_temp));
+                    else
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu->metrics.memory_temp);
+                    ImGui::SameLine(0, 1.0f);
+                    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_temp_fahrenheit])
+                        HUDElements.TextColored(HUDElements.colors.text, "°F");
+                    else
+                        HUDElements.TextColored(HUDElements.colors.text, "°C");
+                }
+
+                if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_clock]){
+                    ImguiNextColumnOrNewRow();
+                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu->metrics.MemClock);
+                    ImGui::SameLine(0, 1.0f);
+                    ImGui::PushFont(HUDElements.sw_stats->font1);
+                    HUDElements.TextColored(HUDElements.colors.text, "MHz");
+                    ImGui::PopFont();
+                }
+                if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_horizontal])
+                    ImGui::TableNextRow();
+                i++;
+            }
         }
     }
 }
@@ -538,7 +563,7 @@ void HudElements::ram(){
         ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", memused);
         if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact]){
-            ImGui::SameLine(0,1.0f);
+            ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
             HUDElements.TextColored(HUDElements.colors.text, "GiB");
             ImGui::PopFont();
@@ -548,7 +573,7 @@ void HudElements::ram(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram] && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_swap]){
         ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", swapused);
-        ImGui::SameLine(0,1.0f);
+        ImGui::SameLine(0, 1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
         HUDElements.TextColored(HUDElements.colors.text, "GiB");
         ImGui::PopFont();
@@ -560,33 +585,34 @@ void HudElements::procmem()
 {
 #ifdef __linux__
     const char* unit = nullptr;
+
     if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem])
         return;
 
     ImguiNextColumnFirstItem();
     HUDElements.TextColored(HUDElements.colors.ram, "PMEM");
     ImguiNextColumnOrNewRow();
-    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.resident, unit));
-    ImGui::SameLine(0,1.0f);
+    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem_resident, unit));
+    ImGui::SameLine(0, 1.0f);
     ImGui::PushFont(HUDElements.sw_stats->font1);
     HUDElements.TextColored(HUDElements.colors.text, "%s", unit);
     ImGui::PopFont();
 
-    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared]){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared]) {
         ImguiNextColumnOrNewRow();
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.shared, unit));
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem_shared, unit));
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
         HUDElements.TextColored(HUDElements.colors.text, "%s", unit);
         ImGui::PopFont();
     }
 
-    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_virt]){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_virt]) {
         ImguiNextColumnOrNewRow();
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.virt, unit));
-        ImGui::SameLine(0,1.0f);
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem_virt, unit));
+        ImGui::SameLine(0, 1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
-       HUDElements.TextColored(HUDElements.colors.text, "%s", unit);
+        HUDElements.TextColored(HUDElements.colors.text, "%s", unit);
         ImGui::PopFont();
     }
 #endif
@@ -745,6 +771,9 @@ static inline double TransformInverse_Custom(double v, void*) {
 
 void HudElements::frame_timing(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_frame_timing]){
+        if (!gpus)
+            gpus = std::make_unique<GPUS>(HUDElements.params);
+
         ImguiNextColumnFirstItem();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_horizontal] && !HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hud_compact]){
@@ -815,11 +844,12 @@ void HudElements::frame_timing(){
                         ImPlot::SetupAxesLimits(0, 200, min_time, max_time);
                         ImPlot::SetNextLineStyle(HUDElements.colors.frametime, 1.5);
                         ImPlot::PlotLine("frametime line", frametime_data.data(), frametime_data.size());
-                        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status_graph] && throttling){
+                        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status_graph] &&
+                            gpus->active_gpu() && gpus->active_gpu()->throttling()){
                             ImPlot::SetNextLineStyle(ImVec4(1.0f, 1.0f, 0.0f, 1.0f), 1.5);
-                            ImPlot::PlotLine("power line", throttling->power.data(), throttling->power.size());
+                            ImPlot::PlotLine("power line", gpus->active_gpu()->throttling()->power.data(), gpus->active_gpu()->throttling()->power.size());
                             ImPlot::SetNextLineStyle(ImVec4(1.0f, 0.0f, 0.0f, 1.0f), 1.5);
-                            ImPlot::PlotLine("thermal line", throttling->thermal.data(), throttling->thermal.size());
+                            ImPlot::PlotLine("thermal line", gpus->active_gpu()->throttling()->thermal.data(), gpus->active_gpu()->throttling()->thermal.size());
                         }
                         ImPlot::EndPlot();
                     }
@@ -829,10 +859,11 @@ void HudElements::frame_timing(){
         }
         ImGui::EndChild();
 #ifdef __linux__
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status_graph] && throttling){
+        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status_graph] &&
+            gpus->active_gpu() && gpus->active_gpu()->throttling()){
             ImGui::Dummy(ImVec2(0.0f, real_font_size.y / 2));
 
-            if (throttling->power_throttling()) {
+            if (gpus->active_gpu()->throttling()->power_throttling()) {
                 ImGui::TextColored(ImVec4(1.0f, 1.0f, 0.0f, 1.0f), "%s", ICON_FK_SQUARE);
                 ImGui::SameLine();
                 ImGui::Text("Power throttling");
@@ -840,7 +871,7 @@ void HudElements::frame_timing(){
 
             ImGui::Dummy(ImVec2(0.0f, real_font_size.y / 2));
 
-            if (throttling->thermal_throttling()) {
+            if (gpus->active_gpu()->throttling()->thermal_throttling()) {
                 ImGui::TextColored(ImVec4(1.0f, 0.0f, 0.0f, 1.0f), "%s", ICON_FK_SQUARE);
                 ImGui::SameLine();
                 ImGui::Text("Thermal throttling");
@@ -942,7 +973,10 @@ void HudElements::_exec(){
     ImguiNextColumnFirstItem();
     for (auto& item : HUDElements.exec_list){
         if (item.pos == HUDElements.place){
-            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", item.ret.c_str());
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_horizontal])
+                HUDElements.TextColored(HUDElements.colors.text, "%s",item.ret.c_str());
+            else
+                right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", item.ret.c_str());
         }
     }
     ImGui::PopFont();
@@ -1089,7 +1123,7 @@ void HudElements::gamescope_frame_timing(){
                                         HUDElements.gamescope_debug_app.end());
             auto max = std::max_element(HUDElements.gamescope_debug_app.begin(),
                                         HUDElements.gamescope_debug_app.end());
-            
+
             ImGui::PushFont(HUDElements.sw_stats->font1);
             ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
             HUDElements.TextColored(HUDElements.colors.engine, "%s", "App");
@@ -1224,18 +1258,25 @@ void HudElements::fan(){
 
 void HudElements::throttling_status(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status]){
-        if (gpu_info.is_power_throttled || gpu_info.is_current_throttled || gpu_info.is_temp_throttled || gpu_info.is_other_throttled){
+        if (!gpus)
+            gpus = std::make_unique<GPUS>(HUDElements.params);
+
+        auto gpu = gpus->active_gpu();
+        if (!gpu)
+            return;
+
+        if ((gpu->metrics.is_power_throttled || gpu->metrics.is_current_throttled || gpu->metrics.is_temp_throttled || gpu->metrics.is_other_throttled)){
             ImguiNextColumnFirstItem();
             HUDElements.TextColored(HUDElements.colors.engine, "%s", "Throttling");
             ImguiNextColumnOrNewRow();
             ImguiNextColumnOrNewRow();
-            if (gpu_info.is_power_throttled)
+            if (gpu->metrics.is_power_throttled)
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Power");
-            if (gpu_info.is_current_throttled)
+            if (gpu->metrics.is_current_throttled)
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Current");
-            if (gpu_info.is_temp_throttled)
+            if (gpu->metrics.is_temp_throttled)
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Temp");
-            if (gpu_info.is_other_throttled)
+            if (gpu->metrics.is_other_throttled)
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Other");
         }
     }
@@ -1338,8 +1379,14 @@ void HudElements::graphs(){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_vram_used));
         }
+        if (!gpus)
+            gpus = std::make_unique<GPUS>(HUDElements.params);
 
-        HUDElements.max = gpu_info.memoryTotal;
+        auto gpu = gpus->active_gpu();
+        if (!gpu)
+            return;
+
+        HUDElements.max = gpu->metrics.memoryTotal;
         HUDElements.min = 0;
         HUDElements.TextColored(HUDElements.colors.engine, "%s", "VRAM");
     }
@@ -1456,16 +1503,15 @@ void HudElements::network() {
         HUDElements.net = std::make_unique<Net>();
 
     for (auto& iface : HUDElements.net->interfaces){
-        ImGui::TableNextRow();
-        ImGui::TableNextColumn();
+        ImguiNextColumnFirstItem();
         HUDElements.TextColored(HUDElements.colors.network, "%.8s", iface.name.c_str());
-        ImGui::TableNextColumn();
+        ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.0f", iface.txBps / 1000.f);
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
         HUDElements.TextColored(HUDElements.colors.text, "KB/s %s", ICON_FK_ARROW_UP);
         ImGui::PopFont();
-        ImGui::TableNextColumn();
+        ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.0f", iface.rxBps / 1000.f);
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -1475,6 +1521,132 @@ void HudElements::network() {
 #endif
 }
 
+void HudElements::_display_session() {
+    ImGui::PushFont(HUDElements.sw_stats->font1);
+    ImguiNextColumnFirstItem();
+    HUDElements.TextColored(HUDElements.colors.engine, "%s", "Display server");
+    ImguiNextColumnOrNewRow();
+    static std::map<display_servers, std::string> servers {
+        {WAYLAND, {"WAYLAND"}},
+        {XWAYLAND, {"XWAYLAND"}},
+        {XORG, {"XORG"}}
+    };
+    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", servers[HUDElements.display_server].c_str());
+    ImGui::PopFont();
+}
+
+void HudElements::fex_stats()
+{
+#ifdef HAVE_FEX
+    if (!HUDElements.params->fex_stats.enabled) {
+        return;
+    }
+
+    ImGui::PushFont(HUDElements.sw_stats->font1);
+
+    if (HUDElements.params->fex_stats.status) {
+        ImguiNextColumnFirstItem();
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "FEX");
+        ImguiNextColumnOrNewRow();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImguiNextColumnOrNewRow();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", fex::fex_status);
+    }
+
+    if (!fex::is_fex_pid_found()) {
+        ImGui::PopFont();
+        return;
+    }
+
+    if (HUDElements.params->fex_stats.app_type) {
+        ImguiNextColumnFirstItem();
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "Type");
+        ImguiNextColumnOrNewRow();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImguiNextColumnOrNewRow();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", fex::get_fex_app_type());
+    }
+
+    if (HUDElements.params->fex_stats.sigbus_counts) {
+        ImguiNextColumnFirstItem();
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "SIGBUS");
+        ImguiNextColumnOrNewRow();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImguiNextColumnOrNewRow();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%d - %.0f avg/s", fex::sigbus_counts.Count(), fex::sigbus_counts.Avg());
+    }
+
+    if (HUDElements.params->fex_stats.smc_counts) {
+        ImguiNextColumnFirstItem();
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "SMC");
+        ImguiNextColumnOrNewRow();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImguiNextColumnOrNewRow();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%d - %.0f avg/s", fex::smc_counts.Count(), fex::smc_counts.Avg());
+    }
+
+    if (HUDElements.params->fex_stats.softfloat_counts) {
+        ImguiNextColumnFirstItem();
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "Softfloat");
+        ImguiNextColumnOrNewRow();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImguiNextColumnOrNewRow();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%d - %.0f avg/s", fex::softfloat_counts.Count(), fex::softfloat_counts.Avg());
+    }
+
+    ImGui::PopFont();
+
+    ImguiNextColumnFirstItem();
+    ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+
+    if (HUDElements.params->fex_stats.hot_threads) {
+        // Draw hot threads
+        bool Warning = false;
+        ImVec4 WarningColor;
+
+        for (auto it : fex::fex_max_thread_loads){
+            if (it >= 75.0) {
+                Warning = true;
+                WarningColor = ImVec4(1.0f, 0.0f, 0.0f, 1.0f);
+            }
+            else if (it >= 50.0) {
+                Warning = true;
+                WarningColor = ImVec4(1.0f, 1.0f, 0.0f, 1.0f);
+            }
+        }
+
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "FEX JIT top loaded threads");
+        ImGui::PopFont();
+
+        ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0f, 0.0f, 0.0f, 0.0f));
+        if (Warning) {
+            ImGui::PushStyleColor(ImGuiCol_PlotHistogram, WarningColor);
+        }
+
+        ImGui::PlotHistogram("", fex::fex_max_thread_loads.data(),
+            fex::fex_max_thread_loads.size(), 0,
+            NULL, 0, 100,
+            ImVec2(ImGui::GetWindowContentRegionWidth(), 50));
+        ImGui::PopStyleColor(1 + (Warning ? 1 : 0));
+    }
+
+    if (HUDElements.params->fex_stats.jit_load) {
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        HUDElements.TextColored(HUDElements.colors.engine, "%s", "FEX JIT Load");
+        ImGui::PopFont();
+
+        ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0f, 0.0f, 0.0f, 0.0f));
+
+        ImGui::PlotLines("", fex::fex_load_data.data(),
+            fex::fex_load_data.size(), 0,
+            NULL, 0, 100,
+            ImVec2(ImGui::GetWindowContentRegionWidth(), 50));
+        ImGui::PopStyleColor(1);
+    }
+#endif //HAVE_FEX
+}
+
 void HudElements::sort_elements(const std::pair<std::string, std::string>& option) {
     const auto& param = option.first;
     const auto& value = option.second;
@@ -1522,8 +1694,9 @@ void HudElements::sort_elements(const std::pair<std::string, std::string>& optio
         {"refresh_rate", {refresh_rate}},
         {"winesync", {winesync}},
         {"present_mode", {present_mode}},
-        {"network", {network}}
-
+        {"network", {network}},
+        {"display_server", {_display_session}},
+        {"fex_stats", {fex_stats}},
     };
 
     auto check_param = display_params.find(param);
@@ -1650,6 +1823,10 @@ void HudElements::legacy_elements(){
         ordered_functions.push_back({present_mode, "present_mode", value});
     if (params->enabled[OVERLAY_PARAM_ENABLED_refresh_rate])
         ordered_functions.push_back({refresh_rate, "refresh_rate", value});
+    if (params->enabled[OVERLAY_PARAM_ENABLED_display_server])
+        ordered_functions.push_back({_display_session, "display_session", value});
+    if (params->fex_stats.enabled)
+        ordered_functions.push_back({fex_stats, "fex_stats", value});
 }
 
 void HudElements::update_exec(){
@@ -1657,8 +1834,12 @@ void HudElements::update_exec(){
     if (!HUDElements.shell)
         HUDElements.shell = std::make_unique<Shell>();
 
-    for(auto& item : exec_list)
-        item.ret = HUDElements.shell->exec(item.value + "\n");
+    for(auto& item : exec_list){
+        std::string ret = HUDElements.shell->exec(item.value + "\n");
+        // use the previous ret if we get bad system call
+        if (ret.find("Bad system call") == std::string::npos)
+            item.ret = ret;
+    }
 #endif
 }
 
