@@ -1,6 +1,7 @@
 #include "iostats.h"
 #include "string_utils.h"
 #include <fstream>
+#include "hud_elements.h"
 
 struct iostats g_io_stats;
 
@@ -11,9 +12,22 @@ void getIoStats(iostats& io) {
     io.prev.read_bytes  = io.curr.read_bytes;
     io.prev.write_bytes = io.curr.write_bytes;
 
-    std::string line;
-    std::ifstream f("/proc/self/io");
-    while (std::getline(f, line)) {
+    std::string f = "/proc/";
+
+    {
+        auto gs_pid = HUDElements.g_gamescopePid;
+        f += gs_pid < 1 ? "self" : std::to_string(gs_pid);
+        f += "/io";
+    }
+
+    std::ifstream file(f);
+
+    if (!file.is_open()) {
+        SPDLOG_ERROR("can't open {}", f);
+        return;
+    }
+
+    for (std::string line; std::getline(file, line);) {
         if (starts_with(line, "read_bytes:")) {
             try_stoull(io.curr.read_bytes, line.substr(12));
         }
