@@ -101,9 +101,8 @@ void render_imgui(swapchain_stats& data, struct overlay_params& params, ImVec2&
 void update_hud_info(struct swapchain_stats& sw_stats, const struct overlay_params& params, uint32_t vendorID);
 void update_hud_info_with_frametime(struct swapchain_stats& sw_stats, const struct overlay_params& params, uint32_t vendorID, uint64_t frametime_ns);
 void update_hw_info(const struct overlay_params& params, uint32_t vendorID);
-void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_params& params);
 void init_cpu_stats(overlay_params& params);
-void check_keybinds(overlay_params& params, uint32_t vendorID);
+void check_keybinds(overlay_params& params);
 void init_system_info(void);
 void FpsLimiter(struct fps_limit& stats);
 void create_fonts(ImFontAtlas* font_atlas, const overlay_params& params, ImFont*& small_font, ImFont*& text_font);
