@@ -7,31 +7,32 @@ extern "C" {
 }
 #include "stdio.h"
 #include "../src/amdgpu.h"
-#include "../src/cpu.h"
 
 #define UNUSED(x) (void)(x)
 
 static void test_amdgpu_verify_metrics(void **state) {
     UNUSED(state);
+    AMDGPU amdgpu("", 0x744c, 0x1002);
 
-    assert_false(amdgpu_verify_metrics("./missing_file"));
+    assert_false(amdgpu.verify_metrics("./missing_file"));
     // unsupported struct size, format and content revision
-    assert_false(amdgpu_verify_metrics("./gpu_metrics_invalid"));
-    assert_true (amdgpu_verify_metrics("./gpu_metrics"));
+    assert_false(amdgpu.verify_metrics("./gpu_metrics_invalid"));
+    assert_true (amdgpu.verify_metrics("./gpu_metrics"));
 }
 
 static void test_amdgpu_get_instant_metrics(void **state) {
     UNUSED(state);
     struct amdgpu_common_metrics metrics;
-
+    AMDGPU amdgpu("", 0x744c, 0x1002);
+    std::string metrics_path;
     // fail fetch gpu_metrics file
     metrics_path = "./missing_file";
-    amdgpu_get_instant_metrics(&metrics);
+    amdgpu.get_instant_metrics(&metrics);
 
     // DGPU
     metrics_path = "./gpu_metrics";
     metrics = {};
-    amdgpu_get_instant_metrics(&metrics);
+    amdgpu.get_instant_metrics(&metrics);
     assert_int_equal(metrics.gpu_load_percent, 64);
     assert_float_equal(metrics.average_gfx_power_w, 33, 0);
     assert_float_equal(metrics.average_cpu_power_w, 0, 0);
@@ -48,7 +49,7 @@ static void test_amdgpu_get_instant_metrics(void **state) {
     // DGPU
     metrics_path = "./gpu_metrics_reserved_throttle_bits";
     metrics = {};
-    amdgpu_get_instant_metrics(&metrics);
+    amdgpu.get_instant_metrics(&metrics);
     assert_false(metrics.is_power_throttled);
     assert_false(metrics.is_current_throttled);
     assert_false(metrics.is_temp_throttled);
@@ -56,7 +57,7 @@ static void test_amdgpu_get_instant_metrics(void **state) {
 
     metrics_path = "./gpu_metrics_apu";
     metrics = {};
-    amdgpu_get_instant_metrics(&metrics);
+    amdgpu.get_instant_metrics(&metrics);
     assert_int_equal(metrics.gpu_load_percent, 100);
     assert_float_equal(metrics.average_gfx_power_w, 6.161, 0);
     assert_float_equal(metrics.average_cpu_power_w, 9.235, 0);
@@ -74,18 +75,20 @@ static void test_amdgpu_get_instant_metrics(void **state) {
 
 static void test_amdgpu_get_samples_and_copy(void **state) {
     UNUSED(state);
+    AMDGPU amdgpu("", 0x744c, 0x1002);
 
     struct amdgpu_common_metrics metrics_buffer[100];
     bool gpu_load_needs_dividing = false;  //some GPUs report load as centipercent
-    amdgpu_get_samples_and_copy(metrics_buffer, gpu_load_needs_dividing);
+    amdgpu.get_samples_and_copy(metrics_buffer, gpu_load_needs_dividing);
     gpu_load_needs_dividing = true;
-    amdgpu_get_samples_and_copy(metrics_buffer, gpu_load_needs_dividing);
+    amdgpu.get_samples_and_copy(metrics_buffer, gpu_load_needs_dividing);
 }
 
 static void test_amdgpu_get_metrics(void **state) {
     UNUSED(state);
+    AMDGPU amdgpu("", 0x744c, 0x1002);
 
-    amdgpu_get_metrics(0x1435);
+    amdgpu.copy_metrics();
 }
 
 const struct CMUnitTest amdgpu_tests[] = {
